package com.neurotrack.mhp.app.bottom_navigation.native_scores.scoreslist

import android.graphics.Typeface
import android.util.TypedValue
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.core.content.ContextCompat
import com.google.android.material.button.MaterialButton
import com.neurotrack.mhp.R
import com.neurotrack.mhp.app.bottom_navigation.native_scores.NativeScoresViewModel
import com.neurotrack.mhp.app.generic_list.TitleStyle
import com.neurotrack.mhp.common.extensions.asDateTime
import com.neurotrack.mhp.common.extensions.asLocaleLongFormat
import com.neurotrack.mhp.common.extensions.inflate
import com.neurotrack.mhp.common.widgets.ScoresProgressBar
import java.text.ParseException
import kotlinx.android.synthetic.main.scores_summary_card_component.view.*


class TestClass {
    fun methodThatExceedsLinelength(parent: ViewGroup, viewType: Int): SomeBaseTypeObject1<out SomeBaseOtherTypeObject1> {
        val rootView = parent.inflate(viewType)
        return when (viewType) {
            1 -> Class1ThatExceedsMaxLineLength(rootView)
            2 -> Class2ThatExceedsMaxLineLength(rootView)
            3 -> Class3ThatExceedsMaxLineLength(rootView)
            4 -> Class4ThatExceedsMaxLineLength(rootView)

            else -> throw IllegalArgumentException("Invalid type $viewType")
        }
    }
}

private class Class3ThatExceedsMaxLineLength(private val root: View) : SomeBaseTypeObject1<SomeMiscellaneousObjectType>(root) {

    override fun bind(item: SomeMiscellaneousObjectType) {
        root.run {

            if (item.icon != 0) {
                findViewById<ImageView>(R.id.cardIcon).setImageDrawable(ContextCompat.getDrawable(context, item.icon))
            }
            findViewById<TextView>(R.id.lastTaken).text = Class2ThatExceedsMaxLineLength.formatDateString(item.cognitiveScoreSummary.lastTaken)
        }
    }
}

private class Class2ThatExceedsMaxLineLength(private val root: View) : SomeBaseTypeObject1<SomeMiscellaneousObjectType2>(root) {
    companion object {

        internal fun formatDateString(rawDate: String): String =
            try {
                val date = rawDate.asDateTime() ?: throw ParseException("failed to parse: $rawDate", 0)
                date.asLocaleLongFormat()
            } catch (ex: ParseException) {
                ex.message ?: ""
            }
    }

    override fun bind(item: SomeMiscellaneousObjectType2) {
        root.run {
            if (item.icon != 0) {
                findViewById<ImageView>(R.id.cardIcon).setImageDrawable(ContextCompat.getDrawable(context, item.icon))
            }
            findViewById<TextView>(R.id.lastTaken).text = formatDateString(item.cognitiveScoreSummary.lastTaken)
        }
    }
}


private class Class4ThatExceedsMaxLineLength(private val root: View) : SomeBaseTypeObject1<SomeMiscellaneousObjectType3>(root) {
    override fun bind(item: SomeMiscellaneousObjectType3) {

        when (item.titleType) {
            TitleStyle.TITLE -> {
                cardTitle.setTextSize(TypedValue.COMPLEX_UNIT_PX, root.resources.getDimensionPixelSize(R.dimen.scores_headline_header).toFloat())
            }
            TitleStyle.HEADLINE -> {
                cardTitle.setTextSize(TypedValue.COMPLEX_UNIT_PX, root.resources.getDimensionPixelSize(R.dimen.scores_domain_header).toFloat())
            }
            TitleStyle.NONE -> {
                cardTitle.visibility = View.GONE
            }
        }
    }
}


private class Class1ThatExceedsMaxLineLength(private val root: View) : SomeBaseTypeObject1<SomeMiscellaneousObjectType4>(root) {
    override fun bind(item: SomeMiscellaneousObjectType4) {

        root.findViewById<MaterialButton>(R.id.viewMore).apply {
            setText(item.compositeScoreSummary.background)
            setOnClickListener { item.onClick(item) }
        }

        if (item.compositeScoreSummary is SomeMiscellaneousObjectType5) {
            compositeScore.setRanges(scoresDetails.scoreBounds, scoresDetails.rawScore) {
                if (scoresDetails.category == NativeScoresViewModel.lifestyleCategory) {
                    scoresDetails.rawScore / 10
                } else scoresDetails.rawScore
            }

            lastUpdated += Class2ThatExceedsMaxLineLength.formatDateString(item.compositeScoreSummary.lastTaken)
        } else {
            compositeScore.visibility = View.GONE
        }
    }
}
